@@ -45,30 +45,22 @@ class Residual(nn.Module):
     """
     Implemenation taken from
     https://github.com/lucidrains/sinkhorn-transformer/blob/master/sinkhorn_transformer/sinkhorn_transformer.py
+    However, I do postnorm instead of prenorm.
     """
     def __init__(self, fn, input_channels=0, output_channels=0):
         super(Residual, self).__init__()
         self.fn = fn
         self.resample = nn.Linear(input_channels, output_channels) if input_channels != output_channels else None
+        self.norm = nn.LayerNorm(output_channels)
 
     def forward(self, tensor, **kwargs):
         if self.resample is not None:
-            return self.resample(tensor) + self.fn(tensor, **kwargs)
-        return tensor + self.fn(tensor, **kwargs)
-
-class PreNorm(nn.Module):
-    """
-    Implemenation taken from
-    https://github.com/lucidrains/sinkhorn-transformer/blob/master/sinkhorn_transformer/sinkhorn_transformer.py
-    """
-    def __init__(self, channels, fn):
-        super(PreNorm, self).__init__()
-        self.fn = fn
-        self.norm = nn.LayerNorm(channels)
-
-    def forward(self, tensor, **kwargs):
+            tensor = self.resample(tensor) + self.fn(tensor, **kwargs)
+            tensor = self.norm(tensor)
+            return tensor
+        tensor = tensor + self.fn(tensor, **kwargs)
         tensor = self.norm(tensor)
-        return self.fn(tensor, **kwargs)
+        return tensor
 
 class PositionalEmbedding(nn.Module):
     """
@@ -171,7 +163,7 @@ class LinearAttentionHead(nn.Module):
         P_bar = Q/torch.sqrt(torch.tensor(self.dim).type(Q.type()))
         if self.causal_mask is not None:
             self.causal_mask = self.causal_mask.to(Q.device)
-            P_bar = P_bar.masked_fill_(~self.causal_mask, -1e10)
+            P_bar = P_bar.masked_fill_(~self.causal_mask, float('-inf'))
         P_bar = P_bar.softmax(dim=-1)
 
         # Only save this when visualizing
@@ -196,7 +188,7 @@ class MHAttention(nn.Module):
     Multihead attention, with each head being a Linformer Head
     This feeds directly into a feed forward head
     """
-    def __init__(self, input_size, dim, channels, dim_k, nhead, dropout, activation, checkpoint_level,
+    def __init__(self, input_size, dim, channels, dim_k, nhead, dropout, checkpoint_level,
             parameter_sharing, E_proj, F_proj, full_attention, causal_mask, w_o_intermediate_dim=None, decoder_mode=False, method="learnable"):
         super(MHAttention, self).__init__()
         self.heads = nn.ModuleList()
@@ -229,7 +221,6 @@ class MHAttention(nn.Module):
         else:
             self.w_o_1 = nn.Linear(dim*nhead, w_o_intermediate_dim)
             self.w_o_2 = nn.Linear(w_o_intermediate_dim, channels)
-        self.activation = get_act(activation)
         self.mh_dropout = nn.Dropout(dropout)
 
     def forward(self, tensor, **kwargs):
@@ -246,8 +237,6 @@ class MHAttention(nn.Module):
             else:
                 head_outputs.append(head(Q,K,V,**kwargs))
         out = torch.cat(head_outputs, dim=-1)
-        if self.activation is not None:
-            out = self.activation(out)
         if self.w_o_intermediate_dim is None:
             out = self.w_o(out)
         else:
@@ -285,9 +274,9 @@ class Linformer(nn.Module):
         # If we want causal but only with the encoder
         causal_enc = gen_causal_mask(input_size, dim_k, full_attention) if (causal and not decoder_mode) else None
 
-        get_attn = lambda attn_channels, curr_dim_k: MHAttention(input_size, head_dim, attn_channels, curr_dim_k, nhead, dropout, activation, checkpoint_level, parameter_sharing, E_proj, E_proj, full_attention, causal_enc, w_o_intermediate_dim, decoder_mode=False, method=method)
-        get_attn_context = lambda attn_channels, curr_dim_k: MHAttention(input_size, head_dim, attn_channels, curr_dim_k, nhead, dropout, activation, checkpoint_level, parameter_sharing, E_proj, E_proj, full_attention, causal_mask, w_o_intermediate_dim, decoder_mode=True, method=method)
-        get_ff = lambda input_channels, output_channels: FeedForward(input_channels, output_channels, dim_ff, dropout_ff)
+        get_attn = lambda attn_channels, curr_dim_k: MHAttention(input_size, head_dim, attn_channels, curr_dim_k, nhead, dropout, checkpoint_level, parameter_sharing, E_proj, E_proj, full_attention, causal_enc, w_o_intermediate_dim, decoder_mode=False, method=method)
+        get_attn_context = lambda attn_channels, curr_dim_k: MHAttention(input_size, head_dim, attn_channels, curr_dim_k, nhead, dropout, checkpoint_level, parameter_sharing, E_proj, E_proj, full_attention, causal_mask, w_o_intermediate_dim, decoder_mode=True, method=method)
+        get_ff = lambda input_channels, output_channels: FeedForward(input_channels, output_channels, dim_ff, dropout_ff, activation)
 
         for index in range(depth):
             input_channels = ff_intermediate if (index != 0 and ff_intermediate is not None) and not decoder_mode else channels
@@ -296,7 +285,7 @@ class Linformer(nn.Module):
             attn_layer = get_attn(input_channels, max(1, dim_k - index*k_reduce_by_layer))
             ff_layer = get_ff(input_channels, output_channels)
 
-            attn_layer, ff_layer = map(lambda res_ch_in, res_ch_out, fn: Residual(PreNorm(input_channels, fn), res_ch_in, res_ch_out), (input_channels, input_channels), (input_channels, output_channels), (attn_layer, ff_layer))
+            attn_layer, ff_layer = map(lambda res_ch_in, res_ch_out, fn: Residual(fn, res_ch_in, res_ch_out), (input_channels, input_channels), (input_channels, output_channels), (attn_layer, ff_layer))
 
             if include_ff:
                 layers.extend([attn_layer, ff_layer])
@@ -309,7 +298,7 @@ class Linformer(nn.Module):
             attn_context = get_attn_context(channels, max(1, dim_k - index*k_reduce_by_layer))
             ff_context = get_ff(channels, channels)
 
-            attn_context, ff_context = map(lambda fn: Residual(PreNorm(channels, fn)), (attn_context, ff_context))
+            attn_context, ff_context = map(lambda fn: Residual(fn, channels, channels), (attn_context, ff_context))
 
             if include_ff:
                 layers.extend([attn_context, ff_context])
